@@ -1,32 +1,31 @@
-/*	$Id: basic.c,v 1.2 2003/04/24 15:11:59 rrt Exp $	*/
+/* Basic movement functions
+   Copyright (c) 1997-2004 Sandro Sigala.
+   Copyright (c) 2003-2004 Reuben Thomas.
+   All rights reserved.
 
-/*
- * Copyright (c) 1997-2001 Sandro Sigala.  All rights reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- *
- * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
- * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
- * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
- * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
- * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
- * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
- * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
- * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- */
+   This file is part of Zile.
+
+   Zile is free software; you can redistribute it and/or modify it under
+   the terms of the GNU General Public License as published by the Free
+   Software Foundation; either version 2, or (at your option) any later
+   version.
+
+   Zile is distributed in the hope that it will be useful, but WITHOUT ANY
+   WARRANTY; without even the implied warranty of MERCHANTABILITY or
+   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+   for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with Zile; see the file COPYING.  If not, write to the Free
+   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
+   02111-1307, USA.  */
+
+/*	$Id: basic.c,v 1.13.2.2 2005/01/26 15:17:21 rrt Exp $	*/
 
 #include "config.h"
 
 #include <assert.h>
+#include <limits.h>
 #include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -34,15 +33,22 @@
 
 #include "zile.h"
 #include "extern.h"
+#include "editfns.h"
+
+/* Goal-column to arrive when `prev/next-line' functions are used.  */
+static int cur_goalc;
 
 DEFUN("beginning-of-line", beginning_of_line)
 /*+
 Move point to beginning of current line.
 +*/
 {
-	cur_wp->pointo = 0;
+	cur_bp->pt = line_beginning_position(uniarg);
 
-	thisflag |= FLAG_HIGHLIGHT_REGION_STAYS;
+	/* Change the `goalc' to the beginning of line for next
+	   `prev/next-line' calls.  */
+	thisflag |= FLAG_DONE_CPCN;
+	cur_goalc = 0;
 
 	return TRUE;
 }
@@ -52,67 +58,76 @@ DEFUN("end-of-line", end_of_line)
 Move point to end of current line.
 +*/
 {
-	cur_wp->pointo = cur_wp->pointp->size;
+	cur_bp->pt = line_end_position(uniarg);
 
-	thisflag |= FLAG_HIGHLIGHT_REGION_STAYS;
+	/* Change the `goalc' to the end of line for next
+	   `prev/next-line' calls.  */
+	thisflag |= FLAG_DONE_CPCN;
+	cur_goalc = INT_MAX;
 
 	return TRUE;
 }
 
 /*
- * Get the current goal column.  Take care of expanding
- * tabulations.
+ * Get the goal column.  Take care of expanding tabulations.
  */
-static int get_goalc(void)
+int get_goalc_bp(Buffer *bp, Point pt)
 {
-	int col = 0, t = cur_bp->tab_width;
-	char *sp = cur_wp->pointp->text, *p = sp;
+	int col = 0, t = bp->tab_width, i;
+	const char *sp = astr_cstr(pt.p->text);
 
-	while (p < sp + cur_wp->pointo) {
-		if (*p == '\t')
+	for (i = 0; i < pt.o; i++) {
+		if (sp[i] == '\t')
 			col |= t - 1;
-		++col, ++p;
+		++col;
 	}
 
 	return col;
 }
 
+int get_goalc_wp(Window *wp)
+{
+	return get_goalc_bp(wp->bp, window_pt(wp));
+}
+
+int get_goalc(void)
+{
+	return get_goalc_bp(cur_bp, cur_bp->pt);
+}
+
 /*
  * Go to the column `goalc'.  Take care of expanding
  * tabulations.
  */
 static void goto_goalc(int goalc)
 {
-	int col = 0, t = cur_bp->tab_width, w;
-	char *sp = cur_wp->pointp->text, *p = sp;
+	int col = 0, t = cur_bp->tab_width, w, i;
+	const char *sp = astr_cstr(cur_bp->pt.p->text);
 
-	while (p < sp + cur_wp->pointp->size) {
-		if (col == goalc)
+	for (i = 0; i < astr_len(cur_bp->pt.p->text); i++) {
+                if (col == goalc)
 			break;
-		else if (*p == '\t') {
+		else if (sp[i] == '\t') {
 			for (w = t - col % t; w > 0; w--)
 				if (++col == goalc)
 					break;
 		} else
 			++col;
-		++p;
 	}
 
-	cur_wp->pointo = p - sp;
+	cur_bp->pt.o = i;
 }
 
-static int cur_goalc;
-
 int previous_line(void)
 {
-	if (cur_wp->pointp->prev != cur_bp->limitp) {
+	if (cur_bp->pt.p->prev != cur_bp->limitp) {
 		thisflag |= FLAG_DONE_CPCN | FLAG_NEED_RESYNC;
 
 		if (!(lastflag & FLAG_DONE_CPCN))
 			cur_goalc = get_goalc();
 
-		cur_wp->pointp = cur_wp->pointp->prev;
-		--cur_wp->pointn;
+		cur_bp->pt.p = cur_bp->pt.p->prev;
+		cur_bp->pt.n--;
 
 		goto_goalc(cur_goalc);
 
@@ -132,30 +147,33 @@ column, or at the end of the line if it is not long enough.
 {
 	int uni;
 
-	thisflag |= FLAG_HIGHLIGHT_REGION_STAYS;
-
 	if (uniarg < 0)
 		return FUNCALL_ARG(next_line, -uniarg);
 
-	for (uni = 0; uni < uniarg; ++uni)
-		if (!previous_line()) {
-			minibuf_error("Beginning of buffer");
-			return FALSE;
-		}
+	if (!bobp()) {
+		for (uni = 0; uni < uniarg; ++uni)
+			if (!previous_line()) {
+				thisflag |= FLAG_DONE_CPCN;
+				FUNCALL(beginning_of_line);
+				break;
+			}
+	}
+	else if (lastflag & FLAG_DONE_CPCN)
+		thisflag |= FLAG_DONE_CPCN;
 
 	return TRUE;
 }
 
 int next_line(void)
 {
-	if (cur_wp->pointp->next != cur_bp->limitp) {
+	if (cur_bp->pt.p->next != cur_bp->limitp) {
 		thisflag |= FLAG_DONE_CPCN | FLAG_NEED_RESYNC;
 
 		if (!(lastflag & FLAG_DONE_CPCN))
 			cur_goalc = get_goalc();
 
-		cur_wp->pointp = cur_wp->pointp->next;
-		++cur_wp->pointn;
+		cur_bp->pt.p = cur_bp->pt.p->next;
+		cur_bp->pt.n++;
 
 		goto_goalc(cur_goalc);
 
@@ -175,16 +193,21 @@ column, or at the end of the line if it is not long enough.
 {
 	int uni;
 
-	thisflag |= FLAG_HIGHLIGHT_REGION_STAYS;
-
 	if (uniarg < 0)
 		return FUNCALL_ARG(previous_line, -uniarg);
 
-	for (uni = 0; uni < uniarg; ++uni)
-		if (!next_line()) {
-			minibuf_error("End of buffer");
-			return FALSE;
-		}
+	if (!eobp()) {
+		for (uni = 0; uni < uniarg; ++uni)
+			if (!next_line()) {
+				int old = cur_goalc;
+				thisflag |= FLAG_DONE_CPCN;
+				FUNCALL(end_of_line);
+				cur_goalc = old;
+				break;
+			}
+	}
+	else if (lastflag & FLAG_DONE_CPCN)
+		thisflag |= FLAG_DONE_CPCN;
 
 	return TRUE;
 }
@@ -195,29 +218,54 @@ column, or at the end of the line if it is not long enough.
  */
 void goto_line(int to_line)
 {
-	if (cur_wp->pointn > to_line)
-		ngotoup(cur_wp->pointn - to_line);
-	else if (cur_wp->pointn < to_line)
-		ngotodown(to_line - cur_wp->pointn);
+	if (cur_bp->pt.n > to_line)
+		ngotoup(cur_bp->pt.n - to_line);
+	else if (cur_bp->pt.n < to_line)
+		ngotodown(to_line - cur_bp->pt.n);
+}
+
+DEFUN("goto-char", goto_char)
+/*+
+Read a number N and move the cursor to character number N.
+Position 1 is the beginning of the buffer.
++*/
+{
+	char *ms;
+	int to_char, count;
+
+	do {
+		if ((ms = minibuf_read("Goto char: ", "")) == NULL)
+			return cancel();
+		if ((to_char = atoi(ms)) < 0)
+			ding();
+	} while (to_char < 0);
+
+	gotobob();
+	for (count = 1; count < to_char; ++count)
+		if (!forward_char())
+			break;
+
+	return TRUE;
 }
 
 DEFUN("goto-line", goto_line)
 /*+
 Move cursor to the beginning of the specified line.
+Line 1 is the beginning of the buffer.
 +*/
 {
 	char *ms;
 	int to_line;
-  
+
 	do {
 		if ((ms = minibuf_read("Goto line: ", "")) == NULL)
 			return cancel();
-		if ((to_line = atoi(ms)) < 1)
+		if ((to_line = atoi(ms)) < 0)
 			ding();
-	} while (to_line < 1);
+	} while (to_line < 0);
 
 	goto_line(to_line - 1);
-	cur_wp->pointo = 0;
+	cur_bp->pt.o = 0;
 
 	return TRUE;
 }
@@ -227,9 +275,7 @@ Move cursor to the beginning of the specified line.
  */
 void gotobob(void)
 {
-	cur_wp->pointp = cur_bp->limitp->next;
-	cur_wp->pointo = 0;
-	cur_wp->pointn = 0;
+	cur_bp->pt = point_min();
 	thisflag |= FLAG_DONE_CPCN | FLAG_NEED_RESYNC;
 }
 
@@ -240,9 +286,6 @@ Move point to the beginning of the buffer; leave mark at previous position.
 {
 	set_mark_command();
 	gotobob();
-
-	thisflag |= FLAG_HIGHLIGHT_REGION_STAYS;
-
 	return TRUE;
 }
 
@@ -251,9 +294,7 @@ Move point to the beginning of the buffer; leave mark at previous position.
  */
 void gotoeob(void)
 {
-	cur_wp->pointp = cur_bp->limitp->prev;
-	cur_wp->pointo = cur_wp->pointp->size;
-	cur_wp->pointn = cur_bp->num_lines;
+	cur_bp->pt = point_max();
 	thisflag |= FLAG_DONE_CPCN | FLAG_NEED_RESYNC;
 }
 
@@ -264,22 +305,19 @@ Move point to the end of the buffer; leave mark at previous position.
 {
 	set_mark_command();
 	gotoeob();
-
-	thisflag |= FLAG_HIGHLIGHT_REGION_STAYS;
-
 	return TRUE;
 }
 
 int backward_char(void)
 {
-	if (cur_wp->pointo > 0) {
-		--cur_wp->pointo;
+	if (!bolp()) {
+		cur_bp->pt.o--;
 
 		return TRUE;
-	} else if (cur_wp->pointp->prev != cur_bp->limitp) {
+	} else if (!bobp()) {
 		thisflag |= FLAG_NEED_RESYNC;
-		cur_wp->pointp = cur_wp->pointp->prev;
-		--cur_wp->pointn;
+		cur_bp->pt.p = cur_bp->pt.p->prev;
+		cur_bp->pt.n--;
 		FUNCALL(end_of_line);
 
 		return TRUE;
@@ -296,8 +334,6 @@ On attempt to pass beginning or end of buffer, stop and signal error.
 {
 	int uni;
 
-	thisflag |= FLAG_HIGHLIGHT_REGION_STAYS;
-
 	if (uniarg < 0)
 		return FUNCALL_ARG(forward_char, -uniarg);
 
@@ -312,20 +348,20 @@ On attempt to pass beginning or end of buffer, stop and signal error.
 
 int forward_char(void)
 {
-	if (cur_wp->pointo < cur_wp->pointp->size) {
-		++cur_wp->pointo;
+	if (!eolp()) {
+		cur_bp->pt.o++;
 
 		return TRUE;
-	} else if (cur_wp->pointp->next != cur_bp->limitp) {
+	} else if (!eobp()) {
 		thisflag |= FLAG_NEED_RESYNC;
-		cur_wp->pointp = cur_wp->pointp->next;
-		++cur_wp->pointn;
+		cur_bp->pt.p = cur_bp->pt.p->next;
+		cur_bp->pt.n++;
 		FUNCALL(beginning_of_line);
 
 		return TRUE;
 	}
-
-	return FALSE;
+	else
+		return FALSE;
 }
 
 DEFUN("forward-char", forward_char)
@@ -336,8 +372,6 @@ On reaching end of buffer, stop and signal error.
 {
 	int uni;
 
-	thisflag |= FLAG_HIGHLIGHT_REGION_STAYS;
-
 	if (uniarg < 0)
 		return FUNCALL_ARG(backward_char, -uniarg);
 
@@ -353,7 +387,7 @@ On reaching end of buffer, stop and signal error.
 int ngotoup(int n)
 {
 	for (; n > 0; n--)
-		if (cur_wp->pointp->prev != cur_bp->limitp)
+		if (cur_bp->pt.p->prev != cur_bp->limitp)
 			FUNCALL(previous_line);
 		else
 			return FALSE;
@@ -364,7 +398,7 @@ int ngotoup(int n)
 int ngotodown(int n)
 {
 	for (; n > 0; n--)
-		if (cur_wp->pointp->next != cur_bp->limitp)
+		if (cur_bp->pt.p->next != cur_bp->limitp)
 			FUNCALL(next_line);
 		else
 			return FALSE;
@@ -377,15 +411,13 @@ int ngotodown(int n)
 
 int scroll_down(void)
 {
-	if (cur_wp->pointn > 0) {
+	if (cur_bp->pt.n > 0) {
 		if (ngotoup(SCROLL_LINES)) {
 			/* XXX */
 			return TRUE;
 		} else
 			return FALSE;
-	} else if (cur_wp->pointo > 0)
-		FUNCALL(beginning_of_line);
-	else {
+	} else {
 		minibuf_error("Beginning of buffer");
 		return FALSE;
 	}
@@ -400,8 +432,6 @@ Scroll text of current window downward near full screen.
 {
 	int uni;
 
-	thisflag |= FLAG_HIGHLIGHT_REGION_STAYS;
-
 	if (uniarg < 0)
 		return FUNCALL_ARG(scroll_up, -uniarg);
 
@@ -414,15 +444,13 @@ Scroll text of current window downward near full screen.
 
 int scroll_up(void)
 {
-	if (cur_wp->pointn < cur_bp->num_lines) {
+	if (cur_bp->pt.n < cur_bp->num_lines) {
 		if (ngotodown(SCROLL_LINES)) {
 			/* XXX */
 			return TRUE;
 		} else
 			return FALSE;
-	} else if (cur_wp->pointo < cur_wp->pointp->size)
-		FUNCALL(end_of_line);
-	else {
+	} else {
 		minibuf_error("End of buffer");
 		return FALSE;
 	}
@@ -437,8 +465,6 @@ Scroll text of current window upward near full screen.
 {
 	int uni;
 
-	thisflag |= FLAG_HIGHLIGHT_REGION_STAYS;
-
 	if (uniarg < 0)
 		return FUNCALL_ARG(scroll_down, -uniarg);
 
