@@ -1,37 +1,37 @@
-/*	$Id: killring.c,v 1.2 2003/04/24 15:11:59 rrt Exp $	*/
+/* Kill ring facility functions
+   Copyright (c) 1997-2004 Sandro Sigala.
+   Copyright (c) 2003-2004 Reuben Thomas.
+   All rights reserved.
 
-/*
- * Copyright (c) 1997-2001 Sandro Sigala.  All rights reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- *
- * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
- * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
- * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
- * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
- * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
- * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
- * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
- * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- */
+   This file is part of Zile.
+
+   Zile is free software; you can redistribute it and/or modify it under
+   the terms of the GNU General Public License as published by the Free
+   Software Foundation; either version 2, or (at your option) any later
+   version.
+
+   Zile is distributed in the hope that it will be useful, but WITHOUT ANY
+   WARRANTY; without even the implied warranty of MERCHANTABILITY or
+   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+   for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with Zile; see the file COPYING.  If not, write to the Free
+   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
+   02111-1307, USA.  */
+
+/*	$Id: killring.c,v 1.11 2004/10/06 16:32:19 rrt Exp $	*/
 
 #include "config.h"
 
 #include <stdarg.h>
 #include <stdlib.h>
 #include <string.h>
+#include <ctype.h>
 
 #include "zile.h"
 #include "extern.h"
+#include "editfns.h"
 
 static char *kill_ring_text;
 static int kill_ring_size;
@@ -75,25 +75,28 @@ static void kill_ring_push_nstring(char *s, size_t size)
 	kill_ring_size += size;
 }
 
-static int kill_line(void)
+static int kill_line(int literally)
 {
-	if (cur_wp->pointo < cur_wp->pointp->size) {
+	if (!eolp()) {
 		if (warn_if_readonly_buffer())
 			return FALSE;
 
-		undo_save(UNDO_INSERT_BLOCK, cur_wp->pointn, cur_wp->pointo,
-			  cur_wp->pointp->size - cur_wp->pointo, 0);
+		undo_save(UNDO_INSERT_BLOCK, cur_bp->pt,
+			  astr_len(cur_bp->pt.p->text) - cur_bp->pt.o, 0);
 		undo_nosave = TRUE;
-		while (cur_wp->pointo < cur_wp->pointp->size) {
-			kill_ring_push(cur_wp->pointp->text[cur_wp->pointo]);
+		while (!eolp()) {
+			kill_ring_push(following_char());
 			FUNCALL(delete_char);
 		}
 		undo_nosave = FALSE;
 
 		thisflag |= FLAG_DONE_KILL;
 
-		return TRUE;
-	} else if (cur_wp->pointp->next != cur_bp->limitp) {
+		if (!literally)
+		  return TRUE;
+	}
+
+	if (cur_bp->pt.p->next != cur_bp->limitp) {
 		if (!FUNCALL(delete_char))
 			return FALSE;
 
@@ -114,16 +117,26 @@ DEFUN("kill-line", kill_line)
 Kill the rest of the current line; if no nonblanks there, kill thru newline.
 +*/
 {
-	int uni;
+	int uni, ret = TRUE;
 
 	if (!(lastflag & FLAG_DONE_KILL))
 		flush_kill_ring();
 
-	for (uni = 0; uni < uniarg; ++uni)
-		if (!kill_line())
-			return FALSE;
+	if (uniarg == 1) {
+		kill_line(FALSE);
+	}
+	else {
+		undo_save(UNDO_START_SEQUENCE, cur_bp->pt, 0, 0);
+		for (uni = 0; uni < uniarg; ++uni)
+			if (!kill_line(TRUE)) {
+				ret = FALSE;
+				break;
+			}
+		undo_save(UNDO_END_SEQUENCE, cur_bp->pt, 0, 0);
+	}
 
-	return TRUE;
+	deactivate_mark();
+	return ret;
 }
 
 DEFUN("kill-region", kill_region)
@@ -139,7 +152,7 @@ the text killed this time appends to the text killed last time
 to make one entry in the kill ring.
 +*/
 {
-	struct region r;
+	Region r;
 
 	if (!(lastflag & FLAG_DONE_KILL))
 		flush_kill_ring();
@@ -156,7 +169,7 @@ to make one entry in the kill ring.
 		 */
 		char *p;
 
-		p = copy_text_block(r.startn, r.starto, r.size);
+		p = copy_text_block(r.start.n, r.start.o, r.size);
 		kill_ring_push_nstring(p, r.size);
 		free(p);
 
@@ -164,13 +177,13 @@ to make one entry in the kill ring.
 	} else {
 		int size = r.size;
 
-		if (cur_wp->pointp != r.startp || r.starto != cur_wp->pointo)
+		if (cur_bp->pt.p != r.start.p || r.start.o != cur_bp->pt.o)
 			FUNCALL(exchange_point_and_mark);
-		undo_save(UNDO_INSERT_BLOCK, cur_wp->pointn, cur_wp->pointo, size, 0);
+		undo_save(UNDO_INSERT_BLOCK, cur_bp->pt, size, 0);
 		undo_nosave = TRUE;
 		while (size--) {
-			if (cur_wp->pointo < cur_wp->pointp->size)
-				kill_ring_push(cur_wp->pointp->text[cur_wp->pointo]);
+			if (!eolp())
+				kill_ring_push(following_char());
 			else
 				kill_ring_push('\n');
 			FUNCALL(delete_char);
@@ -179,7 +192,7 @@ to make one entry in the kill ring.
 	}
 
 	thisflag |= FLAG_DONE_KILL;
-	thisflag &= ~(FLAG_HIGHLIGHT_REGION_STAYS);
+	deactivate_mark();
 
 	return TRUE;
 }
@@ -189,7 +202,7 @@ DEFUN("copy-region-as-kill", copy_region_as_kill)
 Save the region as if killed, but don't kill it.
 +*/
 {
-	struct region r;
+	Region r;
 	char *p;
 
 	if (!(lastflag & FLAG_DONE_KILL))
@@ -200,16 +213,77 @@ Save the region as if killed, but don't kill it.
 
 	calculate_region(&r);
 
-	p = copy_text_block(r.startn, r.starto, r.size);
+	p = copy_text_block(r.start.n, r.start.o, r.size);
 	kill_ring_push_nstring(p, r.size);
 	free(p);
 
 	thisflag |= FLAG_DONE_KILL;
-	thisflag &= ~(FLAG_HIGHLIGHT_REGION_STAYS);
+	deactivate_mark();
 
 	return TRUE;
 }
 
+DEFUN("kill-word", kill_word)
+/*+
+Kill characters forward until encountering the end of a word.
+With argument, do this that many times.
++*/
+{
+	if (!(lastflag & FLAG_DONE_KILL))
+		flush_kill_ring();
+
+	if (warn_if_readonly_buffer())
+		return FALSE;
+
+	push_mark();
+	undo_save(UNDO_START_SEQUENCE, cur_bp->pt, 0, 0);
+	FUNCALL_ARG(mark_word, uniarg);
+	FUNCALL(kill_region);
+	undo_save(UNDO_END_SEQUENCE, cur_bp->pt, 0, 0);
+	pop_mark();
+
+	thisflag |= FLAG_DONE_KILL;
+
+	minibuf_write("");	/* Don't write "Set mark" message.  */
+
+	return TRUE;
+}
+
+DEFUN("backward-kill-word", backward_kill_word)
+/*+
+Kill characters backward until encountering the end of a word.
+With argument, do this that many times.
++*/
+{
+	return FUNCALL_ARG(kill_word, !uniarg? -1: -uniarg);
+}
+
+DEFUN("kill-sexp", kill_sexp)
+/*+
+Kill the sexp (balanced expression) following the cursor.
+With ARG, kill that many sexps after the cursor.
+Negative arg -N means kill N sexps before the cursor.
++*/
+{
+	if (!(lastflag & FLAG_DONE_KILL))
+		flush_kill_ring();
+
+	if (warn_if_readonly_buffer())
+		return FALSE;
+
+	push_mark();
+	undo_save(UNDO_START_SEQUENCE, cur_bp->pt, 0, 0);
+	FUNCALL_ARG(mark_sexp, uniarg);
+	FUNCALL(kill_region);
+	undo_save(UNDO_END_SEQUENCE, cur_bp->pt, 0, 0);
+	pop_mark();
+
+	thisflag |= FLAG_DONE_KILL;
+
+	minibuf_write("");	/* Don't write "Set mark" message.  */
+
+	return TRUE;
+}
 
 DEFUN("yank", yank)
 /*+
@@ -228,11 +302,13 @@ killed OR yanked.  Put point at end, and set mark at beginning.
 
 	set_mark_command();
 
-	undo_save(UNDO_REMOVE_BLOCK, cur_wp->pointn, cur_wp->pointo, kill_ring_size, 0);
+	undo_save(UNDO_REMOVE_BLOCK, cur_bp->pt, kill_ring_size, 0);
 	undo_nosave = TRUE;
 	insert_nstring(kill_ring_text, kill_ring_size);
 	undo_nosave = FALSE;
 
+	deactivate_mark();
+
 	return TRUE;
 }
 
