@@ -1,28 +1,24 @@
-/*	$Id: registers.c,v 1.2 2003/04/24 15:11:59 rrt Exp $	*/
+/* Registers facility functions
+   Copyright (c) 1997-2004 Sandro Sigala.  All rights reserved.
 
-/*
- * Copyright (c) 1997-2001 Sandro Sigala.  All rights reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- *
- * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
- * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
- * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
- * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
- * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
- * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
- * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
- * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- */
+   This file is part of Zile.
+
+   Zile is free software; you can redistribute it and/or modify it under
+   the terms of the GNU General Public License as published by the Free
+   Software Foundation; either version 2, or (at your option) any later
+   version.
+
+   Zile is distributed in the hope that it will be useful, but WITHOUT ANY
+   WARRANTY; without even the implied warranty of MERCHANTABILITY or
+   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+   for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with Zile; see the file COPYING.  If not, write to the Free
+   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
+   02111-1307, USA.  */
+
+/*	$Id: registers.c,v 1.12 2004/10/13 16:04:06 rrt Exp $	*/
 
 #include "config.h"
 
@@ -46,12 +42,13 @@ DEFUN("copy-to-register", copy_to_register)
 Copy region into the user specified register.
 +*/
 {
-	struct region r;
+	Region r;
 	char *p;
 	int reg;
 
 	minibuf_write("Copy to register: ");
-	if ((reg = cur_tp->getkey()) == KBD_CANCEL)
+        term_refresh();
+	if ((reg = term_getkey()) == KBD_CANCEL)
 		return cancel();
 	minibuf_clear();
 	reg %= NUM_REGISTERS;
@@ -61,30 +58,37 @@ Copy region into the user specified register.
 
 	calculate_region(&r);
 
-	p = copy_text_block(r.startn, r.starto, r.size);
+	p = copy_text_block(r.start.n, r.start.o, r.size);
 	if (regs[reg].text != NULL)
 		free(regs[reg].text);
 	regs[reg].text = p;
 	regs[reg].size = r.size;
 
-	cur_wp->bp->markp = NULL;
-
 	return TRUE;
 }
 
+static void insert_register(int reg)
+{
+	undo_save(UNDO_REMOVE_BLOCK, cur_bp->pt, regs[reg].size, 0);
+	undo_nosave = TRUE;
+	insert_nstring(regs[reg].text, regs[reg].size);
+	undo_nosave = FALSE;
+}
+
 DEFUN("insert-register", insert_register)
 /*+
 Insert contents of the user specified register.
 Puts point before and mark after the inserted text.
 +*/
 {
-	int reg;
+	int reg, uni;
 
 	if (warn_if_readonly_buffer())
 		return FALSE;
 
 	minibuf_write("Insert register: ");
-	if ((reg = cur_tp->getkey()) == KBD_CANCEL)
+        term_refresh();
+	if ((reg = term_getkey()) == KBD_CANCEL)
 		return cancel();
 	minibuf_clear();
 	reg %= NUM_REGISTERS;
@@ -94,14 +98,16 @@ Puts point before and mark after the inserted text.
 		return FALSE;
 	}
 
-	undo_save(UNDO_REMOVE_BLOCK, cur_wp->pointn, cur_wp->pointo,
-		  regs[reg].size, 0);
-	undo_nosave = TRUE;
-	insert_nstring(regs[reg].text, regs[reg].size);
-	undo_nosave = FALSE;
-
 	set_mark_command();
 
+	undo_save(UNDO_START_SEQUENCE, cur_bp->pt, 0, 0);
+	for (uni = 0; uni < last_uniarg; ++uni)
+		insert_register(reg);
+	undo_save(UNDO_END_SEQUENCE, cur_bp->pt, 0, 0);
+
+	exchange_point_and_mark();
+	deactivate_mark();
+
 	return TRUE;
 }
 
@@ -109,20 +115,22 @@ static void write_registers_list(va_list ap)
 {
 	unsigned int i, count;
 
+        (void)ap;
 	bprintf("%-8s %8s\n", "Register", "Size");
 	bprintf("%-8s %8s\n", "--------", "----");
 	for (i = count = 0; i < NUM_REGISTERS; ++i)
 		if (regs[i].text != NULL) {
-			char buf[24];
+			astr as = astr_new();
 			++count;
 			if (isprint(i))
-				sprintf(buf, "`%c'", i);
+				astr_afmt(as, "`%c'", i);
 			else
-				sprintf(buf, "`\\%o'", i);
-			bprintf("%-8s %8d\n", buf, regs[i].size);
+				astr_afmt(as, "`\\%o'", i);
+			bprintf("%-8s %8d\n", astr_cstr(as), regs[i].size);
+                        astr_delete(as);
 		}
 	if (!count)
-		bprintf("No register defined\n");
+		bprintf("No registers defined\n");
 }
 
 DEFUN("list-registers", list_registers)
