@@ -1,28 +1,26 @@
-/*	$Id: variables.c,v 1.2 2003/04/24 15:12:00 rrt Exp $	*/
+/* Variables handling functions
+   Copyright (c) 1997-2004 Sandro Sigala.
+   Copyright (c) 2003-2004 Reuben Thomas.
+   All rights reserved.
 
-/*
- * Copyright (c) 1997-2001 Sandro Sigala.  All rights reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- *
- * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
- * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
- * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
- * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
- * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
- * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
- * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
- * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- */
+   This file is part of Zile.
+
+   Zile is free software; you can redistribute it and/or modify it under
+   the terms of the GNU General Public License as published by the Free
+   Software Foundation; either version 2, or (at your option) any later
+   version.
+
+   Zile is distributed in the hope that it will be useful, but WITHOUT ANY
+   WARRANTY; without even the implied warranty of MERCHANTABILITY or
+   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+   for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with Zile; see the file COPYING.  If not, write to the Free
+   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
+   02111-1307, USA.  */
+
+/*	$Id: variables.c,v 1.20 2004/11/14 23:57:55 rrt Exp $	*/
 
 #include "config.h"
 
@@ -41,7 +39,7 @@
  */
 static struct var_entry {
 	char *var;	/* Variable name. */
-	char *fmt;	/* Variable format (boolean, color, etc.). */
+	char *fmt;	/* Variable format (boolean, etc.). */
 	char *val;	/* Default value. */
 } def_vars[] = {
 #define X(zile_var, fmt, val, doc) { zile_var, fmt, val },
@@ -55,35 +53,46 @@ void init_variables(void)
 {
 	struct var_entry *p;
 
-	var_table = htable_new();
+	var_table = htable_new(127);
 
 	for (p = &def_vars[0]; p < &def_vars[sizeof(def_vars) / sizeof(def_vars[0])]; p++)
 		set_variable(p->var, p->val);
 }
 
+static void free_var_iter(hpair *pair, va_list ap)
+{
+        (void)ap;
+        unset_variable(pair->key);
+}
+
 void free_variables(void)
 {
+        htable_foreach(var_table, free_var_iter);
 	htable_delete(var_table);
 }
 
 void set_variable(char *var, char *val)
 {
+	unset_variable(var);
 	htable_store(var_table, var, zstrdup(val));
-
-	/* Force refresh of cached variables. */
-	cur_tp->refresh_cached_variables();
 }
 
 void unset_variable(char *var)
 {
 	char *p = htable_fetch(var_table, var);
-	htable_remove(var_table, var);
 	free(p);
 }
 
 char *get_variable(char *var)
 {
-	return htable_fetch(var_table, var);
+	char *val = htable_fetch(var_table, var);
+#ifdef DEBUG
+	if (!val)
+		ZTRACE(("getting unknown variable `%s'\n", var));
+	else
+		ZTRACE(("getting variable `%s' = `%s'\n", var, val));
+#endif
+	return val;
 }
 
 int is_variable_equal(char *var, char *val)
@@ -99,59 +108,50 @@ int lookup_bool_variable(char *var)
 	if ((p = htable_fetch(var_table, var)) != NULL)
 		return !strcmp(p, "true");
 
-#if 0
-	minibuf_error("Warning: used uninitialized variable `@v%s@@'", var);
-	waitkey(2 * 1000);
+#if DEBUG
+	minibuf_error("Warning: used uninitialized variable `%s'", var);
+	waitkey();
 #endif
 
 	return FALSE;
 }
 
-static historyp make_variable_history(void)
+static void make_var_compl_iter(hpair *pair, va_list ap)
 {
-	alist al;
-	historyp hp;
-	hpair *pair;
-
-	al = htable_list(var_table);
-	hp = new_history(FALSE);
-	for (pair = alist_first(al); pair != NULL; pair = alist_next(al))
-		alist_append(hp->completions, zstrdup(pair->key));
-	alist_delete(al);
-
-	return hp;
+        Completion *cp = va_arg(ap, Completion *);
+        alist_append(cp->completions, zstrdup(pair->key));
 }
 
 char *minibuf_read_variable_name(char *msg)
 {
 	char *ms;
-	historyp hp;
+	Completion *cp = new_completion(FALSE);
 
-	hp = make_variable_history();
+        htable_foreach(var_table, make_var_compl_iter, cp);
 
 	for (;;) {
-		ms = minibuf_read_history(msg, "", hp);
+		ms = minibuf_read_completion(msg, "", cp, NULL);
 
 		if (ms == NULL) {
-			free_history(hp);
+			free_completion(cp);
 			cancel();
 			return NULL;
 		}
 
 		if (ms[0] == '\0') {
-			free_history(hp);
+			free_completion(cp);
 			minibuf_error("No variable name given");
 			return NULL;
 		} else if (get_variable(ms) == NULL) {
-			minibuf_error("Undefined variable name");
-			waitkey(2 * 1000);
+			minibuf_error("Undefined variable name `%s'", ms);
+			waitkey();
 		} else {
 			minibuf_clear();
 			break;
 		}
 	}
 
-	free_history(hp);
+	free_completion(cp);
 
 	return ms;
 }
@@ -163,7 +163,7 @@ static char *get_variable_format(char *var)
 		if (!strcmp(p->var, var))
 			return p->fmt;
 
-	return NULL;
+	return "";
 }
 
 DEFUN("set-variable", set_variable)
@@ -171,7 +171,8 @@ DEFUN("set-variable", set_variable)
 Set a variable value to the user specified value.
 +*/
 {
-	char buf[128], *var, *val, *fmt;
+	char *var, *val, *fmt;
+        astr as = astr_new();
 
 	var = minibuf_read_variable_name("Set variable: ");
 	if (var == NULL)
@@ -179,25 +180,24 @@ Set a variable value to the user specified value.
 
 	/* `tab-width' and `fill-column' are local to buffers. */
 	if (!strcmp(var, "tab-width"))
-		sprintf(buf, "%d", cur_bp->tab_width);
+		astr_afmt(as, "%d", cur_bp->tab_width);
 	else if (!strcmp(var, "fill-column"))
-		sprintf(buf, "%d", cur_bp->fill_column);
+		astr_afmt(as, "%d", cur_bp->fill_column);
 	else
-		strcpy(buf, get_variable(var));
+		astr_cpy_cstr(as, get_variable(var));
 	fmt = get_variable_format(var);
-	if (!strcmp(fmt, "c")) {
-		if ((val = minibuf_read_color("Set %s to value: ", var)) == NULL)
-			return cancel();
-	} else if (!strcmp(fmt, "b")) {
+	if (!strcmp(fmt, "b")) {
 		int i;
 		if ((i = minibuf_read_boolean("Set %s to value: ", var)) == -1)
 			return cancel();
 		val = (i == TRUE) ? "true" : "false";
-	} else { /* Non color, boolean or such fixed value varabile. */
-		if ((val = minibuf_read("Set %s to value: ", buf, var)) == NULL)
+	} else { /* Non boolean or such fixed value variable. */
+		if ((val = minibuf_read("Set %s to value: ", astr_cstr(as), var)) == NULL)
 			return cancel();
 	}
 
+        astr_delete(as);
+
 	/*
 	 * The `tab-width' and `fill-column' variables are local
 	 * to the buffer (automatically becomes buffer-local when
@@ -206,16 +206,16 @@ Set a variable value to the user specified value.
 	if (!strcmp(var, "tab-width")) {
 		int i = atoi(val);
 		if (i < 1) {
-			minibuf_error("Invalid tab-width value `@v%s@@'", val);
-			waitkey(2 * 1000);
+			minibuf_error("Invalid tab-width value `%s'", val);
+			waitkey();
 		} else
 			cur_bp->tab_width = i;
 
 	} else if (!strcmp(var, "fill-column")) {
 		int i = atoi(val);
 		if (i < 2) {
-			minibuf_error("Invalid fill-column value `@v%s@@'", val);
-			waitkey(2 * 1000);
+			minibuf_error("Invalid fill-column value `%s'", val);
+			waitkey();
 		} else
 			cur_bp->fill_column = i;
 	} else
@@ -231,7 +231,7 @@ static int sorter(const void *p1, const void *p2)
 
 static void write_variables_list(va_list ap)
 {
-	windowp old_wp = va_arg(ap, windowp);
+	Window *old_wp = va_arg(ap, Window *);
 	alist al;
 	hpair *pair;
 
@@ -242,8 +242,8 @@ static void write_variables_list(va_list ap)
 	al = htable_list(var_table);
 	alist_sort(al, sorter);
 	for (pair = alist_first(al); pair != NULL; pair = alist_next(al))
-		if (pair->data != NULL)
-			bprintf("%-30s \"%s\"\n", pair->key, pair->data);
+		if (pair->val != NULL)
+			bprintf("%-30s \"%s\"\n", pair->key, pair->val);
 	alist_delete(al);
 
 	bprintf("\nLocal buffer variables:\n\n");
