@@ -1,28 +1,24 @@
-/*	$Id: window.c,v 1.2 2003/04/24 15:12:00 rrt Exp $	*/
+/* Windows-handling functions
+   Copyright (c) 1997-2004 Sandro Sigala.  All rights reserved.
 
-/*
- * Copyright (c) 1997-2001 Sandro Sigala.  All rights reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- *
- * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
- * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
- * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
- * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
- * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
- * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
- * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
- * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- */
+   This file is part of Zile.
+
+   Zile is free software; you can redistribute it and/or modify it under
+   the terms of the GNU General Public License as published by the Free
+   Software Foundation; either version 2, or (at your option) any later
+   version.
+
+   Zile is distributed in the hope that it will be useful, but WITHOUT ANY
+   WARRANTY; without even the implied warranty of MERCHANTABILITY or
+   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+   for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with Zile; see the file COPYING.  If not, write to the Free
+   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
+   02111-1307, USA.  */
+
+/*	$Id: window.c,v 1.11.2.1 2005/01/17 18:02:18 rrt Exp $	*/
 
 #include "config.h"
 
@@ -35,12 +31,12 @@
 #include "zile.h"
 #include "extern.h"
 
-windowp new_window(void)
+Window *new_window(void)
 {
-	windowp wp;
+	Window *wp;
 
-	wp = (windowp)zmalloc(sizeof *wp);
-	memset(wp, 0, sizeof *wp);
+	wp = (Window *)zmalloc(sizeof(Window));
+	memset(wp, 0, sizeof(Window));
 
 	return wp;
 }
@@ -48,8 +44,11 @@ windowp new_window(void)
 /*
  * Free the window allocated memory.
  */
-void free_window(windowp wp)
+void free_window(Window *wp)
 {
+	if (wp->saved_pt)
+		free_marker(wp->saved_pt);
+
 	free(wp);
 }
 
@@ -58,7 +57,7 @@ void free_window(windowp wp)
  */
 void free_windows(void)
 {
-	windowp wp, next;
+	Window *wp, *next;
 
 	for (wp = head_wp; wp != NULL; wp = next) {
 		next = wp->next;
@@ -66,13 +65,38 @@ void free_windows(void)
 	}
 }
 
+/* Set the current window and his buffer as the current buffer.  */
+
+void set_current_window(Window *wp)
+{
+	/* Save buffer's point in a new marker.  */
+	if (cur_wp->saved_pt)
+		free_marker(cur_wp->saved_pt);
+
+	cur_wp->saved_pt = point_marker();
+
+	/* Change the current window.  */
+	cur_wp = wp;
+
+	/* Change the current buffer.  */
+	cur_bp = wp->bp;
+
+	/* Update the buffer point with the window's saved point
+	   marker.  */
+	if (cur_wp->saved_pt) {
+		cur_bp->pt = cur_wp->saved_pt->pt;
+		free_marker(cur_wp->saved_pt);
+		cur_wp->saved_pt = NULL;
+	}
+}
+
 DEFUN("split-window", split_window)
 /*+
 Split current window into two windows, one above the other.
 Both windows display the same buffer now current.
 +*/
 {
-	windowp newwp;
+	Window *newwp;
 
 	/* Windows smaller than 4 lines cannot be split. */
 	if (cur_wp->fheight < 4) {
@@ -89,12 +113,8 @@ Both windows display the same buffer now current.
 	cur_wp->eheight = cur_wp->fheight - 1;
 	if (cur_wp->topdelta >= cur_wp->eheight)
 		recenter(cur_wp);
-	newwp->bp = cur_bp;
-        assert(cur_bp == cur_wp->bp);
-	++newwp->bp->num_windows;
-	newwp->pointp = cur_wp->pointp;
-	newwp->pointn = cur_wp->pointn;
-	newwp->pointo = cur_wp->pointo;
+	newwp->bp = cur_wp->bp;
+	newwp->saved_pt = point_marker();
 	newwp->next = cur_wp->next;
 	cur_wp->next = newwp;
 
@@ -106,23 +126,13 @@ DEFUN("delete-window", delete_window)
 Remove the current window from the screen.
 +*/
 {
-	windowp wp;
+	Window *wp, *del_wp = cur_wp;
 
 	if (cur_wp == head_wp && cur_wp->next == NULL) {
 		minibuf_error("Attempt to delete sole ordinary window");
 		return FALSE;
 	}
 
-	if (--cur_bp->num_windows == 0) {
-		/*
-		 * This is the only window that displays
-		 * the buffer.
-		 */
-		cur_bp->save_pointp = cur_wp->pointp;
-		cur_bp->save_pointn = cur_wp->pointn;
-		cur_bp->save_pointo = cur_wp->pointo;
-	}
-
 	if (cur_wp == head_wp)
 		wp = head_wp = head_wp->next;
 	else
@@ -134,9 +144,12 @@ Remove the current window from the screen.
 
 	wp->fheight += cur_wp->fheight;
 	wp->eheight += cur_wp->eheight + 1;
-	free_window(cur_wp);
-	cur_wp = wp;
-	cur_bp = cur_wp->bp;
+
+	/* Set current window. */
+	set_current_window(wp);
+
+	/* Destroy the window.  */
+	free_window(del_wp);
 
 	return TRUE;
 }
@@ -146,7 +159,7 @@ DEFUN("enlarge-window", enlarge_window)
 Make current window one line bigger.
 +*/
 {
-	windowp wp;
+	Window *wp;
 
 	if (cur_wp == head_wp && cur_wp->next == NULL)
 		return FALSE;
@@ -172,7 +185,34 @@ Make current window one line bigger.
 	return TRUE;
 }
 
-windowp popup_window(void)
+DEFUN("shrink-window", shrink_window)
+/*+
+Make current window one line smaller.
++*/
+{
+	Window *wp;
+
+	if ((cur_wp == head_wp && cur_wp->next == NULL) || cur_wp->fheight < 3)
+		return FALSE;
+
+	if ((wp = cur_wp->next) == NULL) {
+		for (wp = head_wp; wp != NULL; wp = wp->next) {
+			if (wp->next == cur_wp)
+				break;
+		}
+	}
+
+	++wp->fheight;
+	++wp->eheight;
+	--cur_wp->fheight;
+	--cur_wp->eheight;
+	if (cur_wp->topdelta >= cur_wp->eheight)
+		recenter(cur_wp);
+
+	return TRUE;
+}
+
+Window *popup_window(void)
 {
 	if (head_wp->next == NULL) {
 		/* There is only one window on the screen, so split it. */
@@ -193,27 +233,17 @@ DEFUN("delete-other-windows", delete_other_windows)
 Make the selected window fill the screen.
 +*/
 {
-	windowp wp, nextwp;
+	Window *wp, *nextwp;
 
 	for (wp = head_wp; wp != NULL; wp = nextwp) {
 		nextwp = wp->next;
-		if (wp != cur_wp) {
-			if (--wp->bp->num_windows == 0) {
-				/*
-				 * This is the only window that displays
-				 * the buffer.
-				 */
-				wp->bp->save_pointp = wp->pointp;
-				wp->bp->save_pointn = wp->pointn;
-				wp->bp->save_pointo = wp->pointo;
-			}
+		if (wp != cur_wp)
 			free_window(wp);
-		}
 	}
 
-	cur_wp->fwidth = cur_wp->ewidth = cur_tp->width;
+	cur_wp->fwidth = cur_wp->ewidth = termp->width;
 	/* Save space for minibuffer. */
-	cur_wp->fheight = cur_tp->height - 1;
+	cur_wp->fheight = termp->height - 1;
 	/* Save space for status line. */
 	cur_wp->eheight = cur_wp->fheight - 1;
 	cur_wp->next = NULL;
@@ -229,72 +259,39 @@ All windows are arranged in a cyclic order.
 This command selects the window one step away in that order.
 +*/
 {
-	if (cur_wp->next != NULL)
-		cur_wp = cur_wp->next;
-	else
-		cur_wp = head_wp;
-	cur_bp = cur_wp->bp;
-
+	set_current_window((cur_wp->next != NULL) ? cur_wp->next: head_wp);
 	return TRUE;
 }
 
-#if 0
-void new_window_buffer(bufferp bp)
-{
-	if (--cur_bp->num_windows == 0) {
-		/*
-		 * This is the only window that displays
-		 * the buffer.
-		 */
-		cur_bp->pointp = cur_wp->pointp;
-		cur_bp->pointn = cur_wp->pointn;
-		cur_bp->pointo = cur_wp->pointo;
-	}
-
-	cur_wp->bp = cur_bp = bp;
-	++bp->num_windows;
-	cur_wp->pointp = bp->pointp;
-	cur_wp->pointn = bp->pointn;
-	cur_wp->pointo = bp->pointo;
-}
-#endif
-
 /*
  * This function is called once in main(), for creating
  * the scratch buffer.
  */
 void create_first_window(void)
 {
-	windowp wp;
-	bufferp bp;
+	Window *wp;
+	Buffer *bp;
 
 	/*
 	 * Create the scratch buffer.
 	 */
-	bp = new_buffer();
-	set_buffer_name(bp, "*scratch*");
-	bp->flags = BFLAG_NOSAVE | BFLAG_NEEDNAME | BFLAG_TEMPORARY;
-	cur_bp = head_bp = bp;
-	if (lookup_bool_variable("text-mode-auto-fill"))
-		FUNCALL(auto_fill_mode);
+	bp = create_buffer("*scratch*");
+	bp->flags |= BFLAG_NOSAVE | BFLAG_NEEDNAME | BFLAG_TEMPORARY;
+	cur_bp = bp;
 
 	wp = new_window();
 	cur_wp = head_wp = wp;
-	wp->fwidth = wp->ewidth = cur_tp->width;
+	wp->fwidth = wp->ewidth = termp->width;
 	/* Save space for minibuffer. */
-	wp->fheight = cur_tp->height - 1;
+	wp->fheight = termp->height - 1;
 	/* Save space for status line. */
 	wp->eheight = wp->fheight - 1;
 	wp->bp = bp;
-	++bp->num_windows;
-	wp->pointp = bp->save_pointp;
-	wp->pointn = bp->save_pointn;
-	wp->pointo = bp->save_pointo;
 }
 
-windowp find_window(const char *name)
+Window *find_window(const char *name)
 {
-	windowp wp;
+	Window *wp;
 
 	for (wp = head_wp; wp != NULL; wp = wp->next)
 		if (!strcmp(wp->bp->name, name))
@@ -302,3 +299,13 @@ windowp find_window(const char *name)
 
 	return NULL;
 }
+
+Point window_pt(Window *wp)
+{
+	/* The current window uses the current buffer point, all other
+	   windows have a saved point.  */
+	if (wp->saved_pt == NULL)
+                return cur_bp->pt;
+        else
+                return wp->saved_pt->pt;
+}
